// Importe les bibliothèques nécessaires.

#include <Adafruit_NeoPixel.h>

// Définit la configuration générale

#define LED_PIN 6 // Définis la broche sur laquelle est branchée le câble DATA du strip LED
#define LED_COUNT 12 // Définis le nombre de LED par panneau NanoLeaf
#define PANEL_COUNT 10 // Définis le nombre de panneaux NanoLeaf
#define BRIGHTNESS 50 // Définis la luminosité (Min : 0 ; Max : 255)

// Initialise les variables

Adafruit_NeoPixel strip(LED_COUNT * PANEL_COUNT, LED_PIN, NEO_GRBW + NEO_KHZ800); // Crée l'objet strip (nombre de LED, pin DATA, type de strip LED)

void colorFill(uint32_t color, int panel=0); // Crée la fonction permettant de changer immédiatement la couleur des LED
void colorFade(uint32_t color, int steps, int wait, int panel=0); // Crée la fonction permettant de changer progressivement la couleur des LED
void colorWipe(uint32_t color, int wait, int panel=0); // Crée la fonction permettant de changer une par une la couleur des LED

enum colorName {WHITE, BLACK, RED, GREEN, BLUE, YELLOW, CYAN, MAGENTA, GRAY, MAROON, NAVY, OLIVE, PURPLE, TEAL};

// Script

void setup() {
  strip.begin();
  strip.show(); // Eteins toutes les LED de tous les panneaux
  strip.setBrightness(BRIGHTNESS); // Règle la luminosité de tous les panneaux
}

void loop() {
  colorFill(nameToColor(RED)); // Allume en rouge les LED de tous les panneaux
  delay(2000); // Attends 2000 ms

  colorFade(nameToColor(GREEN), 100, 10); // Allume progressivement (100 étapes de 10 ms) en vert les LED de tous les panneaux

  colorWipe(nameToColor(BLUE), 500); // Allume une par une (interval de 500 ms) en bleu les LED de tous les panneaux

  colorFill(nameToColor(YELLOW), 2); // Allume en jaune les LED du panneau n°2
  delay(2000); // Attends 2000 ms

  colorFade(nameToColor(CYAN), 100, 10, 5); // Allume progressivement (100 étapes de 10 ms) en vert les LED du panneau n°5
  
  colorWipe(nameToColor(MAGENTA), 100, 3); // Allume une par une (interval de 100 ms) en magenta les LED du panneau n°3
}

void colorFill(uint32_t color, int panel=0) {
  uint16_t first, count;
    
  if(panel > 0 && panel <= PANEL_COUNT) {
    first = (panel - 1) * LED_COUNT;
    count = LED_COUNT;
  } else { 
    first = 0;
    count = LED_COUNT * PANEL_COUNT;
  }

  strip.fill(color, first, count);
  strip.show(); 
}

void colorWipe(uint32_t color, int wait, int panel=0) {
  int first, count;
  
  if(panel > 0 && panel <= PANEL_COUNT) {
    first = (panel - 1) * LED_COUNT;
    count = LED_COUNT;   
  } else {
    first = 0;
    count = LED_COUNT * PANEL_COUNT;
  }
  
  for(int i = first; i < count; i++) {
    strip.setPixelColor(i, color);
    strip.show();
    delay(wait);
  }
}


void colorFade(uint32_t endColor, int steps, int wait, int panel=0) {
  int first, count;
  
  if(panel > 0 && panel <= PANEL_COUNT) {
    first = (panel - 1) * LED_COUNT;
    count = LED_COUNT;
  } else { 
    first = 0;
    count = LED_COUNT * PANEL_COUNT;
  }
  
  uint32_t startColor = strip.getPixelColor(first);
  uint8_t startRed = (startColor >> 16) & 0xFF;
  uint8_t startGreen = (startColor >> 8) & 0xFF;
  uint8_t startBlue = startColor & 0xFF;

  uint8_t endRed = (endColor >> 16) & 0xFF;
  uint8_t endGreen = (endColor >> 8) & 0xFF;
  uint8_t endBlue = endColor & 0xFF;

  for(int i = 0; i < steps; i++) {
    uint8_t red = startRed + (endRed - startRed) * i / steps;
    uint8_t green = startGreen + (endGreen - startGreen) * i / steps;
    uint8_t blue = startBlue + (endBlue - startBlue) * i / steps;
    
    strip.fill(strip.Color(red, green, blue), first, count);
    strip.show();
    delay(wait);
  }
}

uint32_t nameToColor(colorName color) {
  switch(color) {
    case WHITE:
      return strip.Color(0, 0, 0, 255); // Blanc
      break;
    case BLACK:
      return strip.Color(0, 0, 0, 20); // Noir
      break;
    case RED:
      return strip.Color(255, 0, 0, 10); // Rouge
      break;
    case GREEN:
      return strip.Color(0, 255, 0, 20); // Vert
      break;
    case BLUE:
      return strip.Color(0, 0, 255, 20); // Bleu
      break;
    case YELLOW:
      return strip.Color(255, 255, 0, 10); // Jaune
      break;
    case CYAN:
      return strip.Color(0, 255, 255, 20); // Cyan
      break;
    case MAGENTA:
      return strip.Color(255, 0, 255, 20); // Magenta
      break;
    case GRAY:
      return strip.Color(128, 128, 128); // Gris
      break;
    case MAROON:
      return strip.Color(128, 0, 0, 20); // Marron
      break;
    case NAVY:
      return strip.Color(0, 0, 128); // Bleu marine
      break;
    case OLIVE:
      return strip.Color(128, 128, 0, 20); // Vert Olive
      break;
    case PURPLE:
      return strip.Color(128, 0, 128, 20); // Pourpre
      break;
    case TEAL:
      return strip.Color(0, 128, 128, 20); // Bleu canard
      break;
  }
}
